#ifndef __FARC_H
#define __FARC_H


/* BEGIN HEADER ===============================================================

	 Project:        Arcnet-Treiber for Windows NT
	 Programname:    -
	 Modulname:      -
	 Filename:       FARC.H

	 Description:    Provides constants and types for applications 
                    using the FARC-driver

	 Global Subroutines: -
	 Import:         -
	 Export:         -

	 Current Version:  6.04
	 Created:          05.12.96

	 Company:          SOHARD AG, Fuerth


			(c) SOHARD AG 1996-2005 All rights reserved
** END HEADER   =============================================================*/

/* ------------------ COMPILER CONTROLS  ------------------*/

/* ------------------      INCLUDES       ------------------*/

/* ------------------    DEFINITIONS      ------------------*/

#ifdef TARGET_8051
#define E_FARC_ERROR_CODE_OFFSET           0
#else
#define E_FARC_ERROR_CODE_OFFSET           0xE00503A8L
#endif

#ifndef TARGET_8051
// Operation not successful
// Driver is currently not started
//      CreateFile
#define E_FARC_DRIVER_NOT_STARTED       2L

// invalid handle
// relevant functions: 
//      ReadFile, ReadFileEx, WriteFile, WriteFileEx, DeviceIoControl,
//      CloseFile
#define E_FARC_INVALID_HANDLE            6L

// Read-Operation aborted due to Init/Deinit
// Read-Request returns with invalid buffer
// IOCTL_FARC_GET_NETMAP aborted due to Init/Deinit
// relevant functions: 
//      ReadFile, ReadFileEx, DeviceIoControl IOCTL_FARC_GET_NETMAP
#define E_FARC_ABORT                    995L

// Request pending
#define E_FARC_INCOMPLETE                996L

// Request pending
#define E_FARC_PENDING                   997L
#endif

// wrong parameters in dcb 
// (for Ex.: b_init_behaviour = 4)
#define E_FARC_NO_PARAMETERS            E_FARC_ERROR_CODE_OFFSET + 23L

// read-request or write-request not accepted due to running initialization
// relevant functions: 
//      ReadFile, ReadFileEx, WriteFile, WriteFileEx
#define E_FARC_INITIALIZING             E_FARC_ERROR_CODE_OFFSET + 71L

// read-request or write-request not accepted due to uninitialized Controller
// (DeviceIoControl(IOCTL_FARC_INIT) not yet proceeded)
// relevant functions: 
//      ReadFile, ReadFileEx, WriteFile, WriteFileEx
#define E_FARC_NOT_INITED               E_FARC_ERROR_CODE_OFFSET + 18L

// wrong parameter-mix in dcb:
// f.Ex.: NodeID = 0, 
// or ckp1, ckp2 and ckp3 = TRUE
// or ckp1, ckp3 = TRUE and Clock (in registry) = 20
// relevant functions: 
//      DeviceIoControl
#define E_FARC_BAD_PARAMETER             E_FARC_ERROR_CODE_OFFSET + 87L

// nodeaddress exists
#define E_FARC_DUPID                     E_FARC_ERROR_CODE_OFFSET + 52L

// NAK received (4 or 128 NAKs, depends on initialization)
// relevant functions: 
//      WriteFile, WriteFileEx
#define E_FARC_NAK_TX                    E_FARC_ERROR_CODE_OFFSET + 70L

// no response from receiver
// relevant functions: 
//      WriteFile, WriteFileEx
#define E_FARC_NO_RESPONSE               E_FARC_ERROR_CODE_OFFSET + 55L

// long packets not supported by receiver (20020, 20020C, 20019, 20022 only)
// relevant functions: 
//      WriteFile, WriteFileEx
#define E_FARC_NO_LONG_PACKETS           E_FARC_ERROR_CODE_OFFSET + 56L

// no token seen
// relevant functions: 
//      DeviceIoControl
#define E_FARC_NO_TOKEN                  E_FARC_ERROR_CODE_OFFSET + 8L

// no new-next-id received while initializing
// relevant functions: 
//      DeviceIoControl
#define E_FARC_NO_NEXTID                 E_FARC_ERROR_CODE_OFFSET + 59L

// no bus activity
// relevant functions: 
//      DeviceIoControl
#define E_FARC_NO_ACTIVITY               E_FARC_ERROR_CODE_OFFSET + 29L

// invalid packet-length
//     Write: packet-length (netto) between 253 and 256 or
//            greater than 508 or 
//            less than 0 (less than 7 Bytes in buffer)
//      Read: buffer-size less than 514
// relevant functions: 
//      ReadFile, ReadFileEx, WriteFile, WriteFileEx
#define E_FARC_BAD_PACKET_SIZE           E_FARC_ERROR_CODE_OFFSET + 24L

// IOCTL-number not supported
// relevant functions: 
//      DeviceIoControl
#define E_FARC_DRIVER_OPTION             E_FARC_ERROR_CODE_OFFSET + 1L

// excessive reconfiguration occures, Write-Request/Read-Request denied
// relevant functions: 
//      ReadFile, ReadFileEx, WriteFile, WriteFileEx
#define E_FARC_EXCESSIVE_RECON           E_FARC_ERROR_CODE_OFFSET + 21L

// wrong SID in send-packet
#define E_FARC_BAD_SID                   E_FARC_ERROR_CODE_OFFSET + 37L

// buffer too small
#define E_FARC_BUFFER_TOO_SMALL          E_FARC_ERROR_CODE_OFFSET + 22L

// Init forced with no activity or no token on Bus!!
#define E_FARC_INIT_FORCED               E_FARC_ERROR_CODE_OFFSET + 58L

// No Tx-Pages available, Writerequest denied  
#define E_FARC_WRITE_DENIED              E_FARC_ERROR_CODE_OFFSET + 61L

// more than 5 Read/Write-Requests executed (Demo-Mode only)
#define E_FARC_USAGE_EXPIRED             E_FARC_ERROR_CODE_OFFSET + 30L

// IOCTL_FARC_ENABLE_RECEIVE not allowed. because disable just performing
#define E_FARC_PERFORMING_DISABLE        E_FARC_ERROR_CODE_OFFSET + 31L

// IOCTL_FARC_ENABLE_RECEIVE not allowed. all pages are enabled
// no IOCTL_FARC_DISABLE_RECEIVE previously performed
#define E_FARC_NO_DISABLE                E_FARC_ERROR_CODE_OFFSET + 32L

// no node set for checking in IOCTL_FARC_NETMAP_SETTINGS
#define E_FARC_NETMAP_ERROR				 E_FARC_ERROR_CODE_OFFSET + 45L

// GET_NETMAP just pending
#define E_FARC_NETMAP_PENDING			 E_FARC_ERROR_CODE_OFFSET + 46L

// No netmap change detected
#define E_FARC_NETMAP_NO_CHANGE			 E_FARC_ERROR_CODE_OFFSET + 47L

// Read Timeout occured
#define E_FARC_READ_TIMEOUT				 E_FARC_ERROR_CODE_OFFSET + 34L


#ifndef TARGET_8051
// IOCTL_FARC_XXXX-Constants **********************************************************
// Device Type
#define ARC_TYPE FILE_DEVICE_CONTROLLER

#define IOCTL_FARC_GET_SWVERSION		CTL_CODE(ARC_TYPE, 3,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_CARDID			CTL_CODE(ARC_TYPE, 4,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_INIT					CTL_CODE(ARC_TYPE, 5,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_USEDDCB			CTL_CODE(ARC_TYPE, 6,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_NETSTATE			CTL_CODE(ARC_TYPE, 7,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_NEXTNODEID		CTL_CODE(ARC_TYPE, 8,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_INTERRUPTS		CTL_CODE(ARC_TYPE, 9,  METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_DEINIT				CTL_CODE(ARC_TYPE, 10, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_RXSTATUS			CTL_CODE(ARC_TYPE, 11, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_DISABLE_RECEIVE		CTL_CODE(ARC_TYPE, 12, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_ENABLE_RECEIVE		CTL_CODE(ARC_TYPE, 13, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_EXTNODEID		CTL_CODE(ARC_TYPE, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_EXTND_FUNCTIONS		CTL_CODE(ARC_TYPE, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_CANCEL_IO			CTL_CODE(ARC_TYPE, 16, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_NETMAP_SETTINGS		CTL_CODE(ARC_TYPE, 17, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_NETMAP			CTL_CODE(ARC_TYPE, 18, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_RESOURCE_USAGE   CTL_CODE(ARC_TYPE, 19, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_SET_READ_TIMEOUT 	CTL_CODE(ARC_TYPE, 20, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_FARC_GET_READ_TIMEOUT		CTL_CODE(ARC_TYPE, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif

/* ------------------     CONSTANTS      ------------------*/

/* ------------------ TYPE-DECLARATIONS  ------------------*/

#define eARC_INIT_BREAK 0  // Init will break, if no token seen or if no activity on bus
#define eARC_INIT_FORCE 1  // Init forced, if no token seen or if no activity on bus


// DCB-structure (delivery: IOCTL_FARC_INIT + IOCTL_FARC_GET_USEDDCB)
// sizeof whole struct must be 16 Bytes!!
typedef struct s_ARCNET_DCB {
// Nodeadress of own Node in Arcnet-Network
// Must NOT be 0 !
	unsigned char   b_node_id;

// z_et_1, z_et_2, z_et_3 applies directly to ET1, ET2 und ET3
// in the CONFIG-/SETUP-Register of the arcnetcontroller.
// Sets the responsetime.
// The responsetime of all nodes in the network should be the same.
	unsigned char   z_et_1;
	unsigned char   z_et_2;
	unsigned char   z_et_3;

// z_ckp_1, z_ckp_2, z_ckp_1 applies directly to CKP1, CKP2 und CKP3
// in the SETUP-register of the arcnetcontroller.
// Sets the Bus-Speed. (All 0: 2.5Mbs)
	unsigned char   z_ckp_1;
	unsigned char   z_ckp_2;
	unsigned char   z_ckp_3;

// z_backplane applies directly to BACKPLANE in the CONFIG-register
// of the arcnetcontroller, use only with RS485 Interface
	unsigned char   z_backplane;

// z_p1mode applies directly to P1MODE in the SETUP-register
// of the arcnetcontroller, use only with RS485 Interface
	unsigned char   z_p1mode;

// z_fournaks applies directly to FOURNAKS in the SETUP-register
// of the arcnetcontroller
// If set to 1, it will cause the EXNACK condition to be set after 4 NAKs
// are detected.
// If set to 0, the EXNAK condition will be set after 128 NAKs.
	unsigned char   z_four_naks;

	unsigned char   b_reserved1[2];      // Do NOT use or modify!!

	unsigned char   b_init_behaviour;
// Values: eARC_INIT_BREAK = break init and reset, if no token seen
//         eARC_INIT_FORCE = force init, if no token seen 
//         WARNING: this can cause problems on the net, if connecting to
//                  a running arcnet-Network with a defective arcnet-card)
	unsigned char   b_reserved2[3];      // Do NOT use or modify!!
} ARCNET_DCB;

// Netstate-structure (delivery: IOCTL_FARC_GET_NETSTATE)
typedef struct s_ARCNET_NETSTATE{
	unsigned char	z_token_seen;
	unsigned char	z_net_activity;
	unsigned char	z_txen;
} ARCNET_NETSTATE;

// extended structure (delivery: IOCTL_FARC_EXTND_FUNCTIONS)
typedef struct s_ARCNET_EXTND {
// z_ckup_0, z_ckup_1 applies directly to CKUP0, CKUP1
// in the SETUP2-Register of the arcnetcontroller 20022.
// Sets the Clock multiplier.
	unsigned char   z_ckup_0;
	unsigned char   z_ckup_1;

// z_rcntm_0, z_rcntm_1 applies directly to RCNTM0, RCNTM1
// in the SETUP2-Register of the arcnetcontroller 20022.
	unsigned char   z_rcntm_0;
	unsigned char   z_rcntm_1;

// reserved for future use (must currently set to 0)
	unsigned char   z_receiveall;       // Do NOT use or modify!!
	unsigned char   b_reserved[11];     // Do NOT use or modify!! 
} ARCNET_EXTND;

// Netmap settings/query structure 
// (delivery: IOCTL_FARC_NETMAP_SETTINGS)
// (delivery: IOCTL_FARC_GET_NETMAP)
typedef struct s_ARCNET_NETMAP {
	unsigned char	ba_netmap[32];  // netmap
									// ba_netmap[0]: Node 0..7
									// ba_netmap[1]: Node 8..15
									// ...
									// ba_netmap[31] Node 248..255
									// Bit0 (LSB) corresponds with least significant node
									// i.e.: (ba_netmap[1] & 0x10) addresses node 12
	unsigned short	w_timeout;      // 10ms units
} ARCNET_NETMAP;


/* ------------------ MODULGLOBAL VARIABLES ---------------*/

/* ------------------  EXTERNAL VARIABLES ------------------*/

/* ------------------  EXTERNAL FUNCTIONS ------------------*/

/* ------------------  INTERNAL FUNCTIONS ------------------*/

#endif
