/* BEGIN HEADER ===============================================================

    Project:            Arcnet Driver for Windows NT
    Programname:        NETMAP
    Modulename:
    Filename:           netmap.c

    Description:        Example for netmap usage

    Current Version:    1.1
    Created:            02.05.2003

    Author:             S. Menger
    Company:            SOHARD AG

    Changes:

    Version Date     ch. by Ch-Idx Ch.-Description
    ------- -------- ------ ------ --------
    1.0     02.05.03 MEN           Created
    1.1     04.10.04 MEN           Bugfix (szDriverName size)

			(c) SOHARD AG 2003 All rights reserved
** END HEADER   =============================================================*/

/* ------------------  COMPILER SWITCHES    ------------------*/

/* ------------------      INCLUDES         ------------------*/
#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "winioctl.h"
#include "stdlib.h"
#include "farc.h"

/* ------------------    DEFINITIONS        ------------------*/

#define VERSION "V1.1"

#define ARCDATAMAX 514
/* ------------------     CONSTANTS         ------------------*/

/* ------------------       TYPES           ------------------*/

/* ------------------ MODULE-GLOBAL VARIABLES  ---------------*/

/* ------------------  EXTERNAL VARIABLES   ------------------*/

/* ------------------  EXTERNAL FUNCTIONENS ------------------*/


VOID showUsage(
   VOID
   );


int __cdecl main(
   int argc,
   char **argv
   )
{

   HANDLE      hDriver;
   WORD			wTmp;
   WORD 			f;
   WORD			wNetmapParamCnt;
   WORD			wNetmapTimeout;
   DWORD       dwErrorCode;
   ULONG       dwInterruptCount;
   DWORD       dwBytesReceived;
   DWORD       dwBytesRead;
   ARCNET_DCB  dcb;
   CHAR        szDriverName[256] = "\\\\.\\" ;
   BOOL        zLeaveLoop;
   BOOL			zRet;
   BYTE 			baNetmapNodes[255];
   OVERLAPPED  ovl;
   ARCNET_NETMAP sNetmapIn, sNetmapOut, sNetmapInOrg, sNetmapOutLast;
   char        szVersion[256];


   printf("NETMAP " VERSION ", SOHARD ARCNET driver netmap-sample\n\n");
   if ( argc < 5 )  {
      showUsage();
      return 1;
   }

   if ( (dcb.b_node_id = (unsigned char)atoi( argv[2] )) == 0) { // NodeID from 1st Argument
      showUsage();
      return 1;
   }

   if (sscanf(argv[3], "%u", &wTmp) == 1) {
      wNetmapTimeout = wTmp;
   }
   else {
      printf("Error: Wrong parameter!\n");
      showUsage();
      return 1;
   }

   for (f=0;f<argc-4;f++) {
   	if (sscanf(argv[f+4], "%u", &wTmp) == 1) {
   		baNetmapNodes[f] = (BYTE)wTmp;
      }
      else {
      	printf("Error: Wrong parameter!\n");
	      showUsage();
   	   return 1;
      }
   }
   wNetmapParamCnt = f;

   strcat(szDriverName, argv[1] );

   if ((hDriver = CreateFile(
      szDriverName,                       // Name
      GENERIC_READ | GENERIC_WRITE,       // Access
      0,                                  // ShareMode
      NULL,                               // Security_Attributes
      OPEN_EXISTING,                      // Create
      FILE_FLAG_OVERLAPPED,               // File Attributes
      NULL                                // Template
      )) != INVALID_HANDLE_VALUE ) {

      printf( "Driver opened!\n" );

      // initialize Controller

      // set DCB-Structure
      dcb.z_et_1 = TRUE;
      dcb.z_et_2 = TRUE;
      dcb.z_et_3 = FALSE;
      dcb.z_ckp_1 = FALSE;
      dcb.z_ckp_2 = FALSE;
      dcb.z_ckp_3 = FALSE;
      dcb.z_backplane = FALSE;
      dcb.z_p1mode = FALSE;
      dcb.z_four_naks = FALSE;
      dcb.b_init_behaviour = eARC_INIT_BREAK; //[1]

      memset(&ovl,0,sizeof(OVERLAPPED));
      ovl.hEvent=CreateEvent(NULL,TRUE,FALSE,NULL);
      ResetEvent(ovl.hEvent);

      zRet = DeviceIoControl(
         hDriver,           // Handle
         IOCTL_FARC_INIT,   // Controlcode
         &dcb,              // Input-Buffer
         sizeof(dcb),       // Bufferlength
         NULL,              // Output-Buffer
         0,                 // Bufferlength
         &dwBytesReceived,  // Bytes read
         &ovl               // Pointer to Overlapped Struktur
         );

      if (!zRet) {
         dwErrorCode = GetLastError();
         if (dwErrorCode == E_FARC_PENDING) {
   	  	   zRet = GetOverlappedResult(
                    hDriver,
                    &ovl,
                    &dwBytesRead,
                    TRUE);
            if (!zRet) {
               dwErrorCode = GetLastError();
            }
         }
      }  

      if (!zRet) {
         printf( "Controller-Init fails. Errorcode %lu\n", dwErrorCode );
      }
      else {
         printf( "Controller-Init success.\n" );

         memset (&sNetmapIn.ba_netmap, 0, sizeof(ARCNET_NETMAP));
         sNetmapIn.w_timeout = wNetmapTimeout;
         for (f=0;f<wNetmapParamCnt;f++) {
         	sNetmapIn.ba_netmap[baNetmapNodes[f]/8] |= (1 << (baNetmapNodes[f] % 8));
         }
         memcpy(&sNetmapInOrg, &sNetmapIn, sizeof(ARCNET_NETMAP));

         ResetEvent(ovl.hEvent);
         zRet = DeviceIoControl(
            hDriver,
            IOCTL_FARC_NETMAP_SETTINGS,
            &sNetmapIn,
            sizeof(ARCNET_NETMAP),
            NULL,
            0,
            &dwBytesRead,
            &ovl
            );
            
         if (!zRet) {
            dwErrorCode = GetLastError();
            if (dwErrorCode == E_FARC_PENDING) {
            	zRet = GetOverlappedResult(
                    hDriver,
                    &ovl,
                    &dwBytesRead,
                    TRUE);
               if (!zRet) {
		            dwErrorCode = GetLastError();
               }
            }
         }
         
         if (!zRet) {
          	printf( "IOCTL_FARC_NETMAP_SETTINGS fails. Errorcode %lu\n", dwErrorCode );
         }
         else {
            printf("Netmap checking started. (press any key to abort)\n\n");
            zLeaveLoop = FALSE;
            do {
               ResetEvent(ovl.hEvent);
               zRet = DeviceIoControl(
                  hDriver,
                     IOCTL_FARC_GET_NETMAP,
                  &sNetmapIn,
                  sizeof(ARCNET_NETMAP),
                  &sNetmapOut,
                  sizeof(ARCNET_NETMAP),
                  &dwBytesRead,
                  &ovl
                  );

               if (!zRet) {
                  dwErrorCode = GetLastError();
                  if (dwErrorCode == E_FARC_PENDING) {
                     zRet = GetOverlappedResult(
                          hDriver,
                          &ovl,
                          &dwBytesRead,
		                    TRUE);
      			   	if (!zRet) {
                     	dwErrorCode = GetLastError();
                  	}
                  }
               }

               if (zRet) {
                  // check only wanted nodes
                  for (f=0;f<32;f++) {
                     sNetmapOut.ba_netmap[f] &= sNetmapInOrg.ba_netmap[f];
                  }

                  if (memcmp(&sNetmapOut, &sNetmapOutLast, sizeof(ARCNET_NETMAP))) {
                     memcpy(&sNetmapOutLast, &sNetmapOut, sizeof(ARCNET_NETMAP));
                     memcpy(&sNetmapIn, &sNetmapOutLast, sizeof(ARCNET_NETMAP));

                     printf("Netmap changed:\n");
                     for (f=0;f<255;f++) {
                        if (sNetmapOut.ba_netmap[f/8] & (1 << (f%8))) {
                           printf("%u ", f);
                        }
                     }
                     printf("\n");
                  }
               }
               else {
               	if (dwErrorCode != E_FARC_NETMAP_NO_CHANGE)
                  {
                  	printf( "IOCTL_FARC_GET_NETMAP fails. Errorcode %lu\n", dwErrorCode );
                  }
               }

               if (kbhit()) {
                  getch();
                  zLeaveLoop = TRUE;
                  printf("User break!\n\n");
               }

            } while (!zLeaveLoop);
			}

	      ResetEvent(ovl.hEvent);
         zRet = DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_INTERRUPTS,
            NULL,
            0,
            &dwInterruptCount,
            sizeof( ULONG ),
            &dwBytesReceived,
            &ovl
            );

         if (!zRet) {
            dwErrorCode = GetLastError();
            if (dwErrorCode == E_FARC_PENDING) {
	            zRet = GetOverlappedResult(
                    hDriver,
                    &ovl,
                    &dwBytesRead,
                    TRUE);
         	}
         }

         if (zRet)  {
            printf( "Interrupt counter: %lu\n", dwInterruptCount );
         }

	      ResetEvent(ovl.hEvent);
         zRet = DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_SWVERSION,
            NULL,
            0,
            szVersion,
            sizeof( szVersion ),
            &dwBytesReceived,
            &ovl
            );
         if (!zRet) {
            dwErrorCode = GetLastError();
            if (dwErrorCode == E_FARC_PENDING) {
	            zRet = GetOverlappedResult(
                    hDriver,
                    &ovl,
                    &dwBytesRead,
                    TRUE);
	            dwErrorCode = GetLastError();
            }
         }

         if (zRet) {
            printf( "Driver version: %s\n", szVersion );
         }
         else {
           printf( "Version read failed. Errorcode: %lu\n", dwErrorCode );
         }

	      ResetEvent(ovl.hEvent);
         zRet = DeviceIoControl(
            hDriver,             // Handle
            IOCTL_FARC_DEINIT,   // Controlcode
            NULL,                // Input-Buffer
            0,                   // Bufferlength
            NULL,                // Output-Buffer
            0,                   // Bufferlength
            &dwBytesReceived,    // Bytes read
            &ovl                 // Pointer to Overlapped Struktur
            );

         if (!zRet) {
            dwErrorCode = GetLastError();
            if (dwErrorCode == E_FARC_PENDING) {
             	zRet = GetOverlappedResult(
                       hDriver,
                       &ovl,
                       &dwBytesRead,
                       TRUE);
		         if (!zRet) {
	   	         dwErrorCode = GetLastError();
               }
            }
         }

			if (zRet) {
         	printf( "DEINIT OK\n");
         }
         else {
         	printf( "DEINIT failed. (Errorcode: %lu)\n", dwErrorCode );
         }
      }
      CloseHandle(ovl.hEvent);
      CloseHandle(hDriver);
   }
   else {
       dwErrorCode = GetLastError();
       printf("Can't get a handle to Driver %s. Errorcode: %lu\n",argv[1],dwErrorCode);
   }
   return 0;
}


VOID showUsage(VOID)
{
   printf("Usage: netmap <DriverAccessName> <own Node ID> <Netmap Timeout> <node1> [<node2>]...\n"
          "  e.g. netmap FARC 5 50 1 2 3 4 254\n"
          " the drivers monitors the nodes 1 to 4 and 254 to build a netmap\n"
          " netmap timeout is 500 ms (read documentation)\n"
          " This example requires a running network that consists of at least 2 nodes\n"
          "\nPress any key...\n"
          );
   do ; while (!kbhit());
   getch();
}